@@ -169,7 +169,7 @@ function config_trove_apache_wsgi {
     local trove_apache_conf
 
     sudo mkdir -p ${TROVE_WSGI_DIR}
-    sudo cp $TROVE_DIR/trove/cmd/app.wsgi $TROVE_WSGI_DIR/app.wsgi
+    sudo cp $TROVE_DIR/trove/cmd/app_wsgi.py $TROVE_WSGI_DIR/app_wsgi.py
     trove_apache_conf=$(apache_site_config_for trove-api)
     sudo cp $TROVE_DEVSTACK_FILES/apache-trove-api.template ${trove_apache_conf}
     sudo sed -e "
@@ -180,8 +180,6 @@ function config_trove_apache_wsgi {
         s|%APIWORKERS%|${API_WORKERS}|g;
     " -i ${trove_apache_conf}
     enable_apache_site trove-api
-    tail_log trove-access /var/log/${APACHE_NAME}/trove-api-access.log
-    tail_log trove-api /var/log/${APACHE_NAME}/trove-api.log
 }
 
 # configure_trove() - Set config files, create data dirs, etc
@@ -218,6 +216,7 @@ function configure_trove {
     iniset_conditional $TROVE_CONF DEFAULT usage_timeout $TROVE_USAGE_TIMEOUT
     iniset_conditional $TROVE_CONF DEFAULT state_change_wait_time $TROVE_STATE_CHANGE_WAIT_TIME
     iniset_conditional $TROVE_CONF DEFAULT reboot_time_out 300
+    iniset $TROVE_CONF DEFAULT controller_address ${SERVICE_HOST}
 
     configure_keystone_authtoken_middleware $TROVE_CONF trove
     iniset $TROVE_CONF service_credentials username trove
@@ -237,10 +236,8 @@ function configure_trove {
     iniset $TROVE_CONF DEFAULT taskmanager_manager trove.taskmanager.manager.Manager
     iniset $TROVE_CONF DEFAULT default_datastore $TROVE_DATASTORE_TYPE
 
-    iniset $TROVE_CONF DEFAULT remote_nova_client trove.common.clients_admin.nova_client_trove_admin
-    iniset $TROVE_CONF DEFAULT remote_cinder_client trove.common.clients_admin.cinder_client_trove_admin
-    iniset $TROVE_CONF DEFAULT remote_neutron_client trove.common.clients_admin.neutron_client_trove_admin
-    iniset $TROVE_CONF DEFAULT remote_glance_client trove.common.clients_admin.glance_client_trove_admin
+    iniset $TROVE_CONF cache enabled true
+    iniset $TROVE_CONF cache backend dogpile.cache.memory
 
     iniset $TROVE_CONF cassandra tcp_ports 7000,7001,7199,9042,9160
     iniset $TROVE_CONF couchbase tcp_ports 8091,8092,4369,11209-11211,21100-21199
@@ -275,19 +272,32 @@ function configure_trove {
     iniset $TROVE_GUESTAGENT_CONF service_credentials region_name $REGION_NAME
     iniset $TROVE_GUESTAGENT_CONF service_credentials auth_url $TROVE_AUTH_ENDPOINT
 
-    iniset $TROVE_GUESTAGENT_CONF DEFAULT remote_nova_client trove.common.clients_admin.nova_client_trove_admin
-    iniset $TROVE_GUESTAGENT_CONF DEFAULT remote_cinder_client trove.common.clients_admin.cinder_client_trove_admin
-    iniset $TROVE_GUESTAGENT_CONF DEFAULT remote_neutron_client trove.common.clients_admin.neutron_client_trove_admin
-    iniset $TROVE_GUESTAGENT_CONF DEFAULT remote_glance_client trove.common.clients_admin.glance_client_trove_admin
+    iniset $TROVE_GUESTAGENT_CONF mysql docker_image ${TROVE_DATABASE_IMAGE_MYSQL}
+    iniset $TROVE_GUESTAGENT_CONF mysql backup_docker_image ${TROVE_DATABASE_BACKUP_IMAGE_MYSQL}
+    iniset $TROVE_GUESTAGENT_CONF mariadb docker_image ${TROVE_DATABASE_IMAGE_MARIADB}
+    iniset $TROVE_GUESTAGENT_CONF mariadb backup_docker_image ${TROVE_DATABASE_BACKUP_IMAGE_MARIADB}
+    iniset $TROVE_GUESTAGENT_CONF postgresql docker_image ${TROVE_DATABASE_IMAGE_POSTGRES}
+    iniset $TROVE_GUESTAGENT_CONF postgresql backup_docker_image ${TROVE_DATABASE_BACKUP_IMAGE_POSTGRES}
 
-    # To avoid 'Connection timed out' error of sudo command inside the guest agent
-    CLOUDINIT_PATH=/etc/trove/cloudinit/${TROVE_DATASTORE_TYPE}.cloudinit
-    sudo mkdir -p $(dirname "$CLOUDINIT_PATH")
-    sudo touch "$CLOUDINIT_PATH"
-    sudo tee $CLOUDINIT_PATH >/dev/null <<'EOF'
+    # 1. To avoid 'Connection timed out' error of sudo command inside the guest agent
+    # 2. Config the controller IP address used by guest-agent to download Trove code during initialization (only valid for dev_mode=true).
+    common_cloudinit=/etc/trove/cloudinit/common.cloudinit
+    sudo mkdir -p $(dirname ${common_cloudinit})
+    sudo touch ${common_cloudinit}
+    sudo tee ${common_cloudinit} >/dev/null <<EOF
 #cloud-config
 manage_etc_hosts: "localhost"
+write_files:
+  - path: /etc/trove/controller.conf
+    content: |
+      CONTROLLER=${SERVICE_HOST}
 EOF
+
+    # NOTE(lxkong): Remove this when we support common cloud-init file for all datastores.
+    for datastore in "mysql" "mariadb" "postgresql"
+    do
+        sudo cp ${common_cloudinit} /etc/trove/cloudinit/${datastore}.cloudinit
+    done
 }
 
 # install_trove() - Collect source and prepare
@@ -307,6 +317,9 @@ function install_trove {
     if is_service_enabled horizon; then
         install_trove_dashboard
     fi
+
+    # Fix iptables rules that prevent amqp connections from the devstack box to the guests
+    sudo iptables -D openstack-INPUT -j REJECT --reject-with icmp-host-prohibited || true
 }
 
 # install_trove_dashboard() - Collect source and prepare
@@ -358,7 +371,7 @@ function create_subnet_v6 {
     if [[ -n "$IPV6_PRIVATE_NETWORK_GATEWAY" ]]; then
         subnet_params+="--gateway $IPV6_PRIVATE_NETWORK_GATEWAY "
     fi
-    if [ -n $SUBNETPOOL_V6_ID ]; then
+    if [[ -n $SUBNETPOOL_V6_ID ]]; then
         subnet_params+="--subnet-pool $SUBNETPOOL_V6_ID "
     else
         subnet_params+="--subnet-range $FIXED_RANGE_V6 $ipv6_modes} "
@@ -386,9 +399,11 @@ function setup_mgmt_network() {
     die_if_not_set $LINENO network_id "Failed to create network: $NET_NAME, project: ${PROJECT_ID}"
 
     if [[ "$IP_VERSION" =~ 4.* ]]; then
-        NEW_SUBNET_ID=$(create_mgmt_subnet_v4 ${PROJECT_ID} ${network_id} ${SUBNET_NAME} ${SUBNET_RANGE})
-        # openstack router add subnet $ROUTER_ID $NEW_SUBNET_ID
+        net_subnet_id=$(create_mgmt_subnet_v4 ${PROJECT_ID} ${network_id} ${SUBNET_NAME} ${SUBNET_RANGE})
+        # 'openstack router add' has a bug that cound't show the error message
+        # openstack router add subnet ${ROUTER_ID} ${net_subnet_id} --debug
     fi
+
     # Trove doesn't support IPv6 for now.
 #    if [[ "$IP_VERSION" =~ .*6 ]]; then
 #        NEW_IPV6_SUBNET_ID=$(create_subnet_v6 ${PROJECT_ID} ${network_id} ${IPV6_SUBNET_NAME})
@@ -441,55 +456,65 @@ function create_guest_image {
         return 0
     fi
 
-    echo "Starting to create guest image..."
-
-    TROVE_BRANCH=${TROVE_BRANCH} $DEST/trove/integration/scripts/trovestack \
-      build-image \
-      ${TROVE_DATASTORE_TYPE} \
-      ${TROVE_IMAGE_OS} \
-      ${TROVE_IMAGE_OS_RELEASE} \
-      true
-
-    image_name="trove-${TROVE_IMAGE_OS}-${TROVE_IMAGE_OS_RELEASE}-${TROVE_DATASTORE_TYPE}"
+    image_name="trove-guest-${TROVE_IMAGE_OS}-${TROVE_IMAGE_OS_RELEASE}"
+    mkdir -p $HOME/images
     image_file=$HOME/images/${image_name}.qcow2
-    if [ ! -f ${image_file} ]; then
-        echo "Image file was not found at ${image_file}. Probably it was not created."
-        return 1
+
+    if [[ -n ${TROVE_NON_DEV_IMAGE_URL} ]]; then
+        echo "Downloading guest image from ${TROVE_NON_DEV_IMAGE_URL}"
+        curl -sSL ${TROVE_NON_DEV_IMAGE_URL} -o ${image_file}
+    else
+        echo "Starting to create guest image"
+
+        $DEST/trove/integration/scripts/trovestack \
+          build-image \
+          ${TROVE_IMAGE_OS} \
+          ${TROVE_IMAGE_OS_RELEASE} \
+          true \
+          ${TROVE_IMAGE_OS} \
+          ${image_file}
     fi
 
-    ACTIVE=1
-    INACTIVE=0
+    if [[ ! -f ${image_file} ]]; then
+        echo "Image file was not found at ${image_file}"
+        exit 1
+    fi
 
     echo "Add the image to glance"
-    glance_image_id=$(openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} \
-      --os-project-name service --os-username trove \
-      image create ${TROVE_IMAGE_OS}-${TROVE_IMAGE_OS_RELEASE}-${TROVE_DATASTORE_TYPE} \
-      --disk-format qcow2 --container-format bare --property hw_rng_model='virtio' --file ${image_file} \
+    glance_image_id=$(openstack --os-cloud trove \
+      image create ${image_name} \
+      --disk-format qcow2 --container-format bare \
+      --tag trove \
+      --property hw_rng_model='virtio' \
+      --file ${image_file} \
+      --debug \
       -c id -f value)
+     echo "Glance image ${glance_image_id} uploaded"
 
     echo "Register the image in datastore"
     $TROVE_MANAGE datastore_update $TROVE_DATASTORE_TYPE ""
-    $TROVE_MANAGE datastore_version_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION $TROVE_DATASTORE_TYPE $glance_image_id "" $ACTIVE
+    $TROVE_MANAGE datastore_version_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION $TROVE_DATASTORE_TYPE "" "" 1 --image-tags trove
     $TROVE_MANAGE datastore_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION
 
-    # just for tests
-    $TROVE_MANAGE datastore_version_update "$TROVE_DATASTORE_TYPE" "inactive_version" "manager1" $glance_image_id "" $INACTIVE
-    $TROVE_MANAGE datastore_update Test_Datastore_1 ""
-
-    echo "Add validation rules if available"
-    if [ -f $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json ]; then
+    echo "Add parameter validation rules if available"
+    if [[ -f $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json ]]; then
         $TROVE_MANAGE db_load_datastore_config_parameters "$TROVE_DATASTORE_TYPE" "$TROVE_DATASTORE_VERSION" \
             $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json
     fi
+}
 
-    # To avoid 'Connection timed out' error of sudo command inside the guest agent.
-    CLOUDINIT_PATH=/etc/trove/cloudinit/${TROVE_DATASTORE_TYPE}.cloudinit
-    sudo mkdir -p $(dirname "$CLOUDINIT_PATH")
-    sudo touch "$CLOUDINIT_PATH"
-    sudo tee $CLOUDINIT_PATH >/dev/null <<'EOF'
-#cloud-config
-manage_etc_hosts: "localhost"
-EOF
+function create_registry_container {
+    # running a docker registry container
+    echo "Running a docker registry container..."
+    container=$(sudo docker ps -a --format "{{.Names}}" --filter name=registry)
+    if [ -z $container ]; then
+        sudo docker run -d --net=host -e REGISTRY_HTTP_ADDR=0.0.0.0:4000 --restart=always -v /opt/trove_registry/:/var/lib/registry --name registry registry:2
+        trove_agent_datastore_url=https://tarballs.opendev.org/openstack/trove/images/trove-datastore-registry-master.tar.gz
+        curl -o trove-datastore-registry-master.tar.gz $trove_agent_datastore_url
+        sudo tar -zxvf trove-datastore-registry-master.tar.gz -C /opt/trove_registry/
+        rm -rf trove-datastore-registry-master.tar.gz
+    fi
+    iniset $TROVE_CONF DEFAULT docker_insecure_registries "$TROVE_HOST_GATEWAY:4000"
 }
 
 # Set up Trove management network and make configuration change.
@@ -529,7 +554,9 @@ function config_trove_network {
     if [[ "$shared" == "False" ]]; then
         openstack network set ${PRIVATE_NETWORK_NAME} --share
     fi
-    sudo ip route replace ${IPV4_ADDRS_SAFE_TO_USE} via $ROUTER_GW_IP
+    if [[ -n ${ROUTER_GW_IP} && -n ${IPV4_ADDRS_SAFE_TO_USE} ]]; then
+        sudo ip route replace ${IPV4_ADDRS_SAFE_TO_USE} via $ROUTER_GW_IP
+    fi
 
     # Make sure we can reach the management port of the service VM, this
     # configuration is only for testing purpose. In production, it's
@@ -551,11 +578,12 @@ function config_trove_network {
     openstack network list
     echo "Neutron subnet list:"
     openstack subnet list
+    echo "Neutron router:"
+    openstack router show ${ROUTER_ID} -f yaml
     echo "ip route:"
     sudo ip route
 
     # Now make sure the conf settings are right
-    iniset $TROVE_CONF DEFAULT network_label_regex ""
     iniset $TROVE_CONF DEFAULT ip_regex ""
     iniset $TROVE_CONF DEFAULT black_list_regex ""
     iniset $TROVE_CONF DEFAULT management_networks ${mgmt_net_id}
@@ -582,16 +610,13 @@ function config_nova_keypair {
     fi
 
     echo "Creating Trove management keypair ${TROVE_MGMT_KEYPAIR_NAME}"
-    openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} --os-project-name service --os-username trove \
-      keypair create --public-key ${SSH_DIR}/id_rsa.pub ${TROVE_MGMT_KEYPAIR_NAME}
+    openstack --os-cloud trove keypair create --public-key ${SSH_DIR}/id_rsa.pub ${TROVE_MGMT_KEYPAIR_NAME}
 
     iniset $TROVE_CONF DEFAULT nova_keypair ${TROVE_MGMT_KEYPAIR_NAME}
 }
 
 function config_cinder_volume_type {
-    volume_type=$(openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} \
-      --os-project-name service --os-username trove \
-      volume type list -c Name -f value | awk 'NR==1 {print}')
+    volume_type=$(openstack --os-cloud trove volume type list -c Name -f value | awk 'NR==1 {print}')
 
     iniset $TROVE_CONF DEFAULT cinder_volume_type ${volume_type}
 }
@@ -600,14 +625,12 @@ function config_mgmt_security_group {
     local sgid
 
     echo "Creating Trove management security group."
-    sgid=$(openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} --os-project-name service --os-username trove security group create ${TROVE_MGMT_SECURITY_GROUP} -f value -c id)
+    sgid=$(openstack --os-cloud trove security group create ${TROVE_MGMT_SECURITY_GROUP} -f value -c id)
 
     # Allow ICMP
-    openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} --os-project-name service --os-username trove \
-        security group rule create --proto icmp $sgid
+    openstack --os-cloud trove security group rule create --proto icmp $sgid
     # Allow SSH
-    openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} --os-project-name service --os-username trove \
-        security group rule create --protocol tcp --dst-port 22 $sgid
+    openstack --os-cloud trove security group rule create --protocol tcp --dst-port 22 $sgid
 
     iniset $TROVE_CONF DEFAULT management_security_groups $sgid
 }
@@ -632,6 +655,9 @@ if is_service_enabled trove; then
         config_mgmt_security_group
         config_trove_network
         create_guest_image
+        if [ "$TROVE_ENABLE_LOCAL_REGISTRY" == "True" ] ; then
+            create_registry_container
+        fi
 
         echo_summary "Starting Trove"
         start_trove
